"""Script for running tsunami inundation scenario for Dampier, WA, Australia.

Source data such as elevation and boundary data is assumed to be available in
directories specified by project.py
The output sww file is stored in project.output_run_time_dir 

The scenario is defined by a triangular mesh created from project.polygon,
the elevation data and a simulated tsunami generated with URS code.

Ole Nielsen and Duncan Gray, GA - 2005 and Jane Sexton, Nick Bartzis, GA - 2006
"""

#------------------------------------------------------------------------------
# Import necessary modules
#------------------------------------------------------------------------------

# Standard modules
from os import sep
import os
from os.path import dirname, basename
from os import mkdir, access, F_OK
from shutil import copy
import time
import sys

# Related major packages
from anuga.shallow_water import Domain
from anuga.shallow_water import Dirichlet_boundary
from anuga.shallow_water import File_boundary
from anuga.shallow_water import Reflective_boundary
from anuga.shallow_water import Field_boundary
from Numeric import allclose
from anuga.shallow_water.data_manager import export_grid, create_sts_boundary

from anuga.pmesh.mesh_interface import create_mesh_from_regions
from anuga.shallow_water.data_manager import start_screen_catcher, copy_code_files,store_parameters
#from anuga_parallel.parallel_api import distribute, numprocs, myid, barrier
from anuga_parallel.parallel_abstraction import get_processor_name
from anuga.caching import myhash
from anuga.damage_modelling.inundation_damage import add_depth_and_momentum2csv, inundation_damage
from anuga.fit_interpolate.benchmark_least_squares import mem_usage 
from anuga.utilities.polygon import read_polygon, plot_polygons, polygon_area, is_inside_polygon
from anuga.geospatial_data.geospatial_data import find_optimal_smoothing_parameter
from Scientific.IO.NetCDF import NetCDFFile

# Application specific imports
import project                 # Definition of file names and polygons
numprocs = 1
myid = 0

def run_model(**kwargs):
    

    #------------------------------------------------------------------------------
    # Copy scripts to time stamped output directory and capture screen
    # output to file
    #------------------------------------------------------------------------------
    print "Processor Name:",get_processor_name()

    #copy script must be before screen_catcher
    #print kwargs

    print 'output_dir',kwargs['output_dir']
    if myid == 0:
        copy_code_files(kwargs['output_dir'],__file__, 
                 dirname(project.__file__)+sep+ project.__name__+'.py' )

        store_parameters(**kwargs)

   # barrier()

    start_screen_catcher(kwargs['output_dir'], myid, numprocs)

    print "Processor Name:",get_processor_name()

    
    #-----------------------------------------------------------------------
    # Domain definitions
    #-----------------------------------------------------------------------

    # Read in boundary from ordered sts file
    urs_bounding_polygon=create_sts_boundary(os.path.join(project.boundaries_dir,project.scenario_name))

    # Reading the landward defined points, this incorporates the original clipping
    # polygon minus the 100m contour
    landward_bounding_polygon = read_polygon(project.boundaries_dir+'landward_boundary_polygon.txt')

    # Combine sts polyline with landward points
    bounding_polygon = urs_bounding_polygon + landward_bounding_polygon
    
    # counting segments
    N = len(urs_bounding_polygon)-1
    boundary_tags={'back': [N+2,N+3], 'side': [N,N+1,N+4],'ocean': range(N)}

    print 'boundary tags',boundary_tags
       
    #--------------------------------------------------------------------------
    # Create the triangular mesh based on overall clipping polygon with a
    # tagged
    # boundary and interior regions defined in project.py along with
    # resolutions (maximal area of per triangle) for each polygon
    #--------------------------------------------------------------------------

    #IMPORTANT don't cache create_mesh_from_region and Domain(mesh....) as it
    # causes problems with the ability to cache set quantity which takes alot of times
    if myid == 0:
    
        print 'start create mesh from regions'

        create_mesh_from_regions(bounding_polygon,
                             boundary_tags=boundary_tags,
                             maximum_triangle_area=project.res_poly_all,
                             interior_regions=project.interior_regions,
                             filename=project.meshes_dir_name+'.msh',
                             use_cache=True,
                             verbose=True)
   # barrier()

##        covariance_value,alpha = find_optimal_smoothing_parameter (data_file= kwargs['elevation_file'],
##                                alpha_list=[0.001, 0.01, 0.1, 0.15, 0.2, 0.25, 0.3, 0.4, 0.5],
##                                mesh_file = project.meshes_dir_name+'.msh')
##        print 'optimal alpha', covariance_value,alpha       

    #-------------------------------------------------------------------------
    # Setup computational domain
    #-------------------------------------------------------------------------
    print 'Setup computational domain'

    domain = Domain(project.meshes_dir_name+'.msh', use_cache=False, verbose=True)
    print 'memory usage before del domain',mem_usage()
       
    print domain.statistics()
    print 'triangles',len(domain)
    
    kwargs['act_num_trigs']=len(domain)


    #-------------------------------------------------------------------------
    # Setup initial conditions
    #-------------------------------------------------------------------------
    if myid == 0:

        print 'Setup initial conditions'

        from polygon import Polygon_function
        #following sets the stage/water to be offcoast only
        IC = Polygon_function( [(project.poly_mainland, 0)], default = kwargs['tide'],
                                 geo_reference = domain.geo_reference)
        domain.set_quantity('stage', IC)
        #domain.set_quantity('stage',kwargs['tide'] )
        domain.set_quantity('friction', kwargs['friction']) 
        
        print 'Start Set quantity',kwargs['elevation_file']

        domain.set_quantity('elevation', 
                            filename = kwargs['elevation_file'],
                            use_cache = False,
                            verbose = True,
                            alpha = kwargs['alpha'])
        print 'Finished Set quantity'
    #barrier()

##    #------------------------------------------------------
##    # Create x,y,z file of mesh vertex!!!
##    #------------------------------------------------------
##        coord = domain.get_vertex_coordinates()
##        depth = domain.get_quantity('elevation')
##        
##        # Write vertex coordinates to file
##        filename=project.vertex_filename
##        fid=open(filename,'w')
##        fid.write('x (m), y (m), z(m)\n')
##        for i in range(len(coord)):
##            pt=coord[i]
##            x=pt[0]
##            y=pt[1]
##            z=depth[i]
##            fid.write('%.6f,%.6f,%.6f\n' %(x, y, z))
##

    #------------------------------------------------------
    # Distribute domain to implement parallelism !!!
    #------------------------------------------------------

    if numprocs > 1:
        domain=distribute(domain)

    #------------------------------------------------------
    # Set domain parameters
    #------------------------------------------------------
    print 'domain id', id(domain)
    domain.set_name(kwargs['aa_scenario_name'])
    domain.set_datadir(kwargs['output_dir'])
    domain.set_default_order(2) # Apply second order scheme
    domain.set_minimum_storable_height(0.01) # Don't store anything less than 1cm
    domain.set_store_vertices_uniquely(False)
    domain.set_quantities_to_be_stored(['stage', 'xmomentum', 'ymomentum'])
    domain.tight_slope_limiters = 1
    print 'domain id', id(domain)

    #-------------------------------------------------------------------------
    # Setup boundary conditions 
    #-------------------------------------------------------------------------
    print 'Available boundary tags', domain.get_boundary_tags()
    print 'domain id', id(domain)
   
    boundary_urs_out=project.boundaries_dir_name

    Br = Reflective_boundary(domain)
    Bd = Dirichlet_boundary([kwargs['tide'],0,0])
    
    print 'Available boundary tags', domain.get_boundary_tags()
    Bf = Field_boundary(boundary_urs_out+'.sts',  # Change from file_boundary
                   domain, mean_stage= project.tide,
                   time_thinning=1,
                   default_boundary=Bd,
                   use_cache=True,
                   verbose = True,
                   boundary_polygon=bounding_polygon)

    domain.set_boundary({'back': Br,
                         'side': Bd,
                         'ocean': Bf}) 

    kwargs['input_start_time']=domain.starttime

    print'finish set boundary'

    #----------------------------------------------------------------------------
    # Evolve system through time
    #--------------------------------------------------------------------
    t0 = time.time()

    for t in domain.evolve(yieldstep = project.yieldstep, finaltime = kwargs['finaltime']
                       ,skip_initial_step = False): 
        domain.write_time()
        domain.write_boundary_statistics(tags = 'ocean')

            
    x, y = domain.get_maximum_inundation_location()
    q = domain.get_maximum_inundation_elevation()

    print 'Maximum runup observed at (%.2f, %.2f) with elevation %.2f' %(x,y,q)

    print 'That took %.2f seconds' %(time.time()-t0)

    #kwargs 'completed' must be added to write the final parameters to file
    kwargs['completed']=str(time.time()-t0)
    
    if myid==0:
        store_parameters(**kwargs)
   # barrier
    
    print 'memory usage before del domain1',mem_usage()
    
    
#-------------------------------------------------------------
if __name__ == "__main__":
    
    kwargs={}
    kwargs['est_num_trigs']=project.trigs_min
    kwargs['num_cpu']=numprocs
    kwargs['host']=project.host
    kwargs['res_factor']=project.res_factor
    kwargs['starttime']=project.starttime
    kwargs['yieldstep']=project.yieldstep
    kwargs['finaltime']=project.finaltime
    
    kwargs['output_dir']=project.output_run_time_dir
    kwargs['elevation_file']=project.combined_dir_name+'.pts'
    kwargs['file_name']=project.home+'detail.csv'
    kwargs['aa_scenario_name']=project.scenario_name
    kwargs['ab_time']=project.time
    kwargs['res_factor']= project.res_factor
    kwargs['tide']=project.tide
    kwargs['user']=project.user
    kwargs['alpha'] = project.alpha
    kwargs['friction']=project.friction
    kwargs['time_thinning'] = project.time_thinning
    kwargs['dir_comment']=project.dir_comment
    kwargs['export_cellsize']=project.export_cellsize
    

    run_model(**kwargs)
      
       #barrier
